<?php

/*
 * This file is part of the NelmioApiDocBundle package.
 *
 * (c) Nelmio
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Nelmio\ApiDocBundle\PropertyDescriber;

use Nelmio\ApiDocBundle\Describer\ModelRegistryAwareInterface;
use Nelmio\ApiDocBundle\Describer\ModelRegistryAwareTrait;
use Nelmio\ApiDocBundle\Model\Model;
use Nelmio\ApiDocBundle\OpenApiPhp\Util;
use OpenApi\Annotations as OA;
use OpenApi\Generator;
use Symfony\Component\PropertyInfo\Type;

class ObjectPropertyDescriber implements PropertyDescriberInterface, ModelRegistryAwareInterface
{
    use ModelRegistryAwareTrait;

    public function describe(array $types, OA\Schema $property, array $groups = null, ?OA\Schema $schema = null)
    {
        $type = new Type(
            $types[0]->getBuiltinType(),
            false,
            $types[0]->getClassName(),
            $types[0]->isCollection(),
            // BC layer for symfony < 5.3
            method_exists($types[0], 'getCollectionKeyTypes') ? $types[0]->getCollectionKeyTypes() : $types[0]->getCollectionKeyType(),
            method_exists($types[0], 'getCollectionValueTypes') ?
                ($types[0]->getCollectionValueTypes()[0] ?? null) :
                $types[0]->getCollectionValueType()
        ); // ignore nullable field

        if ($types[0]->isNullable()) {
            $weakContext = Util::createWeakContext($property->_context);
            $property->nullable = true;
            $property->oneOf = [new OA\Schema(['ref' => $this->modelRegistry->register(new Model($type, $groups)), '_context' => $weakContext])];

            return;
        }

        $property->ref = $this->modelRegistry->register(new Model($type, $groups));

        if (!$type->isNullable() && null !== $schema) {
            $propertyName = Util::getSchemaPropertyName($schema, $property);
            if (null === $propertyName) {
                return;
            }

            $existingRequiredFields = Generator::UNDEFINED !== $schema->required ? $schema->required : [];
            $existingRequiredFields[] = $propertyName;

            $schema->required = array_values(array_unique($existingRequiredFields));
        }
    }

    public function supports(array $types): bool
    {
        return 1 === count($types) && Type::BUILTIN_TYPE_OBJECT === $types[0]->getBuiltinType();
    }
}
